%--------------------------------------------------------------------------
% Function which uses datastructure NetListData that has been created by
% reading in a Spice-like network listing. (For example, the function
% ReadCir could be used. The NetListData data structure has the
% following form:
%       NetListData.types : cell array of types 
%       NetListData.names : cell array of names
%       NetListData.nodes : matrix of node numbers
%       NetListData.parameters : array of element values
% From the netlist data the function MNA_matrices returns the
% MNA matrices T and S and the source vector w. If the unknowns are
% in the vector x then the s-domain matrix to be solved is
%          (sS + T) x = w
% where s is the Laplace-domain variable.
%--------------------------------------------------------------------------
function [S, T, w] = MNA_matrices(NetListData)

%determine the number of branches in the circuit
numBranches = length(NetListData.types);

%determine size of T matrix (assume Tx = w)
%first find number of nodes in circuit
numNodes = length(unique(NetListData.nodes)) - 1;

%second determine the number of auxiliary equations
numAux = 0;

%check each branch for its type and if stamp dictates that an auxiliary 
%current will be added then add 1 (or 2 depending on the stamp) to numAux.
for branch = 1:1:numBranches
    if(strcmpi(char(NetListData.types{branch}),'SC') || ...
        strcmpi(char(NetListData.types{branch}),'L') || ...
        strcmpi(char(NetListData.types{branch}),'VVT') || ...
        strcmpi(char(NetListData.types{branch}),'CCT') || ...
        strcmpi(char(NetListData.types{branch}),'VS') || ...
        strcmpi(char(NetListData.types{branch}),'AMP'))
        numAux = numAux + 1;
    elseif(strcmpi(char(NetListData.types{branch}),'CVT'))
        numAux = numAux + 2;
    end %if        
end %for

Tsize = numNodes + numAux;  %size of T matrix
T = zeros(Tsize);           %initialize T matrix with zeros.  
                            %You can declare it sparse here if you want
S = zeros(Tsize);           %initialize matrix for s-domain parameters
w = zeros(Tsize, 1);        %initialize right-hand-side vector w
auxindex = numNodes+1;      %initialize index for auxiliary rows and columns

%Loop over each branch checking its type and then using stamp rules
%accordingly to fill T and w.  The loop index variable is "branch".
for branch = 1:1:numBranches
    %get the node numbers for branch and store into nodes vector.
    %nodes will contain the four node numbers for branch as given in the
    %netlist file.
    nodes = NetListData.nodes(branch,:);
    j = nodes(1);
    j_p = nodes(2);
    k = nodes(3);
    k_p = nodes(4);
    
    % get the parameter for the branch
    parameter = NetListData.parameters(branch);
    
    %Here we look at the type of element for branch with a case block
    %and then apply appropriate stamp rules.  The voltage source stamp is
    %given below as an example.
    switch NetListData.types{branch}
        case 'VS'            
            %place ones and minus ones in T matrix as dictated by the
            %voltage source stamp.
            if(j ~= 0)
                T(auxindex,j) = 1;
                T(j,auxindex) = 1;
            end %if
            if(j_p ~= 0)
                T(auxindex,j_p) = -1;            
                T(j_p,auxindex) = -1;
            end %if

            %place the voltage source parameters in the source vector w as
            %required by the voltage source stamp.
            w(auxindex) = parameter;            
            auxindex = auxindex + 1;    %increment the index for auxiliary rows and columns

        case 'IS'

        case 'SC'

        case 'OC'

        case 'G'
            %place G and -G in T matrix as dictated by the
            %admittance stamp
                G = parameter;
                if j ~= 0 && j_p ~= 0
                    T(j,j) =+ G;            
                    T(j,j_p) =+ -G;
                    T(j_p,j) =+ -G;            
                    T(j_p,j_p) =+ G;
                end
                if j == 0 
                   T(j_p,j_p) = T(j_p,j_p) + G;
                end
                if j_p == 0 
                   T(j,j) = T(j,j) + G;
                end

        case 'R'

        case 'C'

        case 'L'

        case 'VCT'

        case 'VVT'

        case 'CCT'

        case 'CVT'

        case 'AMP'

        otherwise
            %An error message will be printed out if the type given in the
            %NetListData.types is not a "case" above.
            errmsg = sprintf('Error: Invalid type given in file for branch %d: %s', branch, NetListData.types{branch});
            disp(errmsg);
    end %switch
end %for branch
















